function [Aq1,Abq1,wwq1,linte,ldire,nove]=stiffq1(ifro,nov,xy,nove,nvle);
% STIFFQ1 Constructs local stiffness Q1 matrices on extended elements for Schwarz
%
%  [Aq1,Abq1,wwq1,linte,ldire,nove]=stiffq1(ifro,nov,xy,nove,nvle);
%
% called by schwarz_2d.m and by eig_schwarz_2d.m
%
% Input: 
%        ifro = column array of length noe=nov(npdx*npdy,ne):
%            if (x_i,y_i) is internal to Omega then ifro(i)=0,
%            if (x_i,y_i) is on \partial\Omega then ifro(i)=1,
%        nov = local -global map, previously generated by cosnov_2d
%        xy = column array with global mesh, length: noe=nov(npdx*npdy,ne)
%        nove = 2-index array of "extended local" to global map, 
%        nvle = 2-index array:
%                 nvle (:,1)=number of nodes of the extendend elements
%                 nvle (:,2)=number of Q1 elements of the extendend elements
%                            along x-direction
%                 nvle (:,3)=number of Q1 elements of the extendend elements
%                            along y-direction
%
% Output: Aq1 = cell array with Q1 stiffness matrices (internal/internal nodes) 
%               on extended elements
%         Abq1 = cell array with Q1 stiffness matrices (internal/boundary nodes) 
%               on extended elements
%         wwq1 = cell array with Q1 mass matrices (internal nodes) 
%                on extended elements
%         linte = cell array with list of internal nodes of extended
%                 elements
%         ldire = cell array with list of boundary nodes of extended
%                 elements
%         nove = 2-index array of "extended local" to global map, permuted.
%
% References: CHQZ2 = C. Canuto, M.Y. Hussaini, A. Quarteroni, T.A. Zang,
%                    "Spectral Methods. Fundamentals in Single Domains"
%                    Springer Verlag, Berlin Heidelberg New York, 2006.
%             CHQZ3 = C. Canuto, M.Y. Hussaini, A. Quarteroni, T.A. Zang,
%                    "Spectral Methods. Evolution to Complex Geometries 
%                     and Applications to Fluid DynamicsSpectral Methods"
%                    Springer Verlag, Berlin Heidelberg New York, 2007.

%   Written by Paola Gervasio
%   $Date: 2007/04/01$

[ldnov,ne]=size(nov);

Aq1=cell(ne,1); Abq1=cell(ne,1);
wwq1=cell(ne,1);
linte=cell(ne,1);
ldire=cell(ne,1);
permu=cell(ne,1);

% loop on extended spectral elements
for ie=1:ne
mne=nvle(ie,1); nexl=nvle(ie,2);neyl=nvle(ie,3); nel=nexl*neyl;
npxl=nexl+1; npyl=neyl+1;
xyl=zeros(mne,2);
xyl(:,1)=xy(nove(1:mne,ie),1);
xyl(:,2)=xy(nove(1:mne,ie),2);

% xyl and nove are sorted following lexicographic ordering

[p]=reorder(xyl,nexl,neyl);
xyl(:,1)=xyl(p,1);xyl(:,2)=xyl(p,2);
nove(1:mne,ie)=nove(p,ie);
% 
ifroe=zeros(mne,1);
ifroe(1:npxl)=1;ifroe(1:npxl:mne)=1;ifroe(npxl:npxl:mne)=1;
ifroe(mne-npxl+1:mne)=1;

permu(ie)={p};

% Structures for Q1 stiffness matrices

x=[-1;1]; wx=[1;1]; dx=[-0.5,0.5;-0.5,0.5];
y=x;wy=wx;dy=dx;
iparl=zeros(10,1);
iparl(7)=mne;iparl(6)=nel;iparl(1)=2;iparl(2)=2;iparl(3)=4;
iparl(4)=nexl;
[novl,jaclx,jacly]=meshq1_ie(nove(:,ie),nvle(ie,:),xyl,iparl);

% stiffness Q1 matrix on the extended element
[Al,wwl]=stiffq1_se(iparl,ifroe,novl,wx,dx,jaclx,wy,dy,jacly);

% filtering  of both stiffness and mass matrices

[listaint,listadir]=liste2(ifroe);
Alb=Al(listaint,listadir);
Al=Al(listaint,listaint);
wwl=wwl(listaint);

Aq1(ie)={chol(Al)}; Abq1(ie)={Alb}; wwq1(ie)={wwl};
linte(ie)={listaint};
ldire(ie)={listadir};
clear Al;
end

