function[xx,yy,jacx,jacy,xy,ww,ifro]=mesh2d(xa,xb,ya,yb,cb,nex,ney,npdx,npdy,...
nov,x,wx,y,wy,gammax,gammay)
% MESH2D   Uniform 2D SEM mesh on rectangular domain Omega=(xa,xb) x (ya,yb)
%
%                side 3
%           V4 __________ V3
%             |          |
%             |          |
%     side 4  | Omega    |  side 2
%             |          |
%             |__________|
%           V1            V2
%                side 1
%
%  [xx,yy,jacx,jacy,xy,ww,ifro,xn]=mesh2d(xa,xg,ya,yb,nex,ney,npdx,npdy,...
%           nov,x,wx,y,wy)
%
% Input: xa= abscissa of either vertex V1 or vertex V4
%        xb= abscissa of either vertex V2 or vertex V3
%        ya= ordinate of either vertex V1 or vertex V2
%        yb= ordinate of either vertex V3 or vertex V4
%        cb= a string of four elements, each for any side of \partial\Omega.
%          cb(i) refers to side number "i" of \partial\Omega
%          'd' character stands for Dirichlet boundary conditions on that side
%          'n' character stands for Neumann boundary conditions on that side
%        nex = number of elements along x-direction
%        ney = number of elements along y-direction
%        npdx = number of nodes in each element (the same in every element)
%               along x-direction
%        npdy = number of nodes in each element (the same in every element)
%               along y-direction
%        nov = local -global map, previously generated by cosnov_2d
%        x = npdx LGL nodes in [-1,1], previously generated by xwlgl
%        wx = npdx LGL weigths in [-1,1], previously generated by xwlgl
%        y = npdy LGL nodes in [-1,1], previously generated by xwlgl
%        wy = npdy LGL weigths in [-1,1], previously generated by xwlgl
%        gammax = column or row array of length nex-1. 
%               If the deomposition of Omega is not
%               uniform, gammax is the vector of position of interfaces between
%               spectral elements along x-direction. If gammax=[], uniform
%               decomposition is used.
%        gammay = column or row array of length ney-1. 
%               If the deomposition of Omega is not
%               uniform, gammay is the vector of position of interfaces between
%               spectral elements along x-direction. If gammay=[], uniform
%               decomposition is used.
%
% Output: xx = 2-indexes array of size (4,ne) 
%            xx(1:4,ie)=[x_V1_ie;x_V2_ie;x_V3_ie;x_V4_ie]
%            (ne=nex*ney is the global number of spectral elements)
%         yy = 2-indexes array of size (4,ne):
%            yy(1:4,ie)=[y_V1_ie;y_V2_ie;y_V3_ie;y_V4_ie]
%         jacx = array (length(jacx)=ne); jacx(ie)= (x_V2_ie-x_V1_ie)/2
%         jacy = array (length(jacy)=ne); jacy(ie)= (y_V3_ie-y_V1_ie)/2
%         xy = column array with global mesh, length: noe=nov(npdx*npdy,ne)
%         ww = column array with global weigths, length: noe=nov(npdx*npdy,ne)
%              diag(ww) is the mass matrix associated to SEM discretization
%         ifro = column array of length noe=nov(npdx*npdy,ne): 
%            if (x_i,y_i) is internal to Omega then ifro(i)=0,
%            if (x_i,y_i) is on \partial\Omega and Dirichlet boundary
%            condition is imposed there, then ifro(i)=1,
%            if (x_i,y_i) is on \partial\Omega and Neumann boundary
%            condition is imposed there, then ifro(i)=31,
%
%
% Reference: CHQZ2 = C. Canuto, M.Y. Hussaini, A. Quarteroni, T.A. Zang,
%                    "Spectral Methods. Fundamentals in Single Domains"
%                    Springer Verlag, Berlin Heidelberg New York, 2006.

%   Written by Paola Gervasio
%   $Date: 2007/04/01$

ldnov=npdx*npdy; ne=nex*ney;
noe=nov(ldnov,ne); nx=npdx-1;
xx=zeros(4,ne); yy=zeros(4,ne); 
xy=zeros(noe,2); ww=zeros(noe,1);
jac=zeros(ne,1);
jacx=zeros(ne,1);
jacy=zeros(ne,1);
if sum(size(gammax))==0 
    Hx=(xb-xa)/nex*ones(nex);
else 
    Hx=zeros(nex,1);
    Hx(1)=gammax(1)-xa;
    Hx(2:nex-1)=gammax(2:nex-1)-gammax(1:nex-2);
    Hx(nex)=xb-gammax(nex-1);
end
if sum(size(gammay))==0 
    Hy=(yb-ya)/ney*ones(ney);
else 
    Hy=zeros(ney,1);
    Hy(1)=gammay(1)-ya;
    Hy(2:ney-1)=gammay(2:ney-1)-gammay(1:ney-2);
    Hy(ney)=yb-gammay(ney-1);
end

for iex=1:nex
ie1=(iex-1)*ney+1;
xx(1,ie1)=xa+sum(Hx(1:iex-1));
xx(2,ie1)=xx(1,ie1)+Hx(iex);
xx(3,ie1)=xx(2,ie1); xx(4,ie1)=xx(1,ie1);
jacx(ie1)=Hx(iex)*.5;
for iey=2:ney
ie=ie1-1+iey; xx(:,ie)=xx(:,ie1);
jacx(ie)=Hx(iex)*.5;
end
end


for iey=1:ney
yy(1,iey)=ya+sum(Hy(1:iey-1));
yy(2,iey)=yy(1,iey);
yy(3,iey)=yy(1,iey)+Hy(iey); yy(4,iey)=yy(3,iey);
jacy(iey)=Hy(iey)*.5;
for iex=2:nex
ie=(iex-1)*ney+iey; yy(:,ie)=yy(:,iey);
jacy(ie)=Hy(iey)*.5;
end
end


for ie=1:ne
bpa=(xx(1,ie)+xx(2,ie))*.5; bpc=(yy(1,ie)+yy(3,ie))*.5; 

for j=1:npdy
for i=1:npdx
k=(j-1)*npdx+i; ii=nov(k,ie);
xy(ii,1)=x(i)*jacx(ie)+bpa; xy(ii,2)=y(j)*jacy(ie)+bpc;
ww(ii)=ww(ii)+wx(i)*jacx(ie)*wy(j)*jacy(ie); 
end
end

end


ifro=zeros(noe,1);

% Loop on elements
for ie=1:ne

% side 1
for i=1:npdx
k=i; ii=nov(k,ie); 
if(abs(xy(ii,2)-ya)<=1.e-12 ) 
if cb(1)=='d'
ifro(ii)=1;
elseif cb(1)=='n'
ifro(ii)=31;
end
end
end

% side 2
for j=1:npdy
k=j*npdx; ii=nov(k,ie); 
if(abs(xy(ii,1)-xb)<=1.e-12 ) 
if cb(2)=='d'
ifro(ii)=1;
elseif cb(2)=='n'
ifro(ii)=31;
end
end
end

% side 3
nm=npdx*(npdy-1);
for i=1:npdx
k=i+nm; ii=nov(k,ie); 
if(abs(xy(ii,2)-yb)<=1.e-12 ) 
if cb(3)=='d'
ifro(ii)=1;
elseif cb(3)=='n'
ifro(ii)=31;
end
end
end

% side 4
for j=1:npdy
k=(j-1)*npdx+1; ii=nov(k,ie);
if(abs(xy(ii,1)-xa)<=1.e-12 )
if cb(4)=='d'
ifro(ii)=1;
elseif cb(4)=='n'
ifro(ii)=31;
end
end
end
% check vertices: if vertex V belongs to two sides with different (N/D)
%boundary conditions, dirichlet b.c. prevails.

% V1
ip=nov(1,ie);
if ifro(ip)>0 & cb(1)~=cb(4)
ifro(ip)=1;
end
% V2
ip=nov(npdx,ie);
if ifro(ip)>0 & cb(1)~=cb(2)
ifro(ip)=1;
end
% V3
ip=nov(ldnov,ie);
if ifro(ip)>0& cb(2)~=cb(3)
ifro(ip)=1;
end
% V4
ip=nov(ldnov-nx,ie);
if ifro(ip)>0& cb(4)~=cb(3)
ifro(ip)=1;
end


% end loop for ie=1:ne
end
